
#pragma once

namespace Box2DLib
{
	DEFINE_ENUM( b2JointType )
	{
		DEFINE_ENUMITEM( e_unknownShape ),
		DEFINE_ENUMITEM( e_unknownJoint ),
		DEFINE_ENUMITEM( e_revoluteJoint ),
		DEFINE_ENUMITEM( e_prismaticJoint ),
		DEFINE_ENUMITEM( e_distanceJoint ),
		DEFINE_ENUMITEM( e_pulleyJoint ),
		DEFINE_ENUMITEM( e_mouseJoint ),
		DEFINE_ENUMITEM( e_gearJoint )
	};

	DEFINE_ENUM( b2LimitState )
	{
		DEFINE_ENUMITEM( e_inactiveLimit ),
		DEFINE_ENUMITEM( e_atLowerLimit ),
		DEFINE_ENUMITEM( e_atUpperLimit ),
		DEFINE_ENUMITEM( e_equalLimits )
	};

	DEFINE_STRUCT( b2Jacobian )
	{
		DEFINE_STRUCT_SET( b2Jacobian );

		DEFINE_PROPERTY_B( b2Vec2, linear1 );
		DEFINE_PROPERTY_V( float32, angular1 );
		DEFINE_PROPERTY_B( b2Vec2, linear2 );
		DEFINE_PROPERTY_V( float32, angular2 );

		DEFINE_METHOD( N, void, SetZero, N, () );
		DEFINE_METHOD( N, void, Set, BVBV, (b2Vec2, float32, b2Vec2, float32) );
		DEFINE_METHOD( V, float32, Compute, BVBV, (b2Vec2, float32, b2Vec2, float32) );
	};

	DEFINE_STRUCT( b2JointDef )
	{
		DEFINE_BASEACCESS( b2JointDef )
		DEFINE_CONSTRUCTOR_DEFAULTDUMMY( b2JointDef )
		DEFINE_DELETEBASE_DUMMY()

		DEFINE_PROPERTY_E( b2JointType, type );
		DEFINE_PROPERTY_C( IntPtr, userData, static_cast<IntPtr>, static_cast<void*> );
		DEFINE_PROPERTY_V_CODE( int, userDataInt, { return reinterpret_cast<int>(BASEMEMBER(userData)); }, { BASEMEMBER(userData) = reinterpret_cast<void *>(v); } );
		DEFINE_PROPERTY_P( b2Body, body1 );
		DEFINE_PROPERTY_P( b2Body, body2 );
		DEFINE_PROPERTY_V( bool, collideConnected );
	};

	DEFINE_STRUCT( b2Joint )
	{
		DEFINE_CONSTRUCTOR_WRAPPER( b2Joint );
		DEFINE_BASEACCESS( b2Joint );
		DEFINE_COPYMETHOD( b2Joint )
		DEFINE_DELETEBASE_DUMMY();

		DEFINE_METHOD( C, MNAME(b2JointType), GetType, N, () );

		DEFINE_METHOD( P, b2Body, GetBody1, N, () );
		DEFINE_METHOD( P, b2Body, GetBody2, N, () );

		virtual DEFINE_METHOD( B, b2Vec2, GetAnchor1, N, () );
		virtual DEFINE_METHOD( B, b2Vec2, GetAnchor2, N, () );

		virtual DEFINE_METHOD( B, b2Vec2, GetReactionForce, N, () );
		virtual DEFINE_METHOD( V, float32, GetReactionTorque, N, () );

		DEFINE_METHOD( P, b2Joint, GetNext, N, () );
		DEFINE_METHOD( C, IntPtr, GetUserData, N, () );
	};

}
