#pragma once

namespace Box2DLib
{
	DEFINE_STRUCT( b2Vec2 )
	{
		DEFINE_STRUCT_SET( b2Vec2 )
		DEFINE_CONSTRUCTOR( b2Vec2, VV, (float32, float32) )

		DEFINE_METHOD( N, void, SetZero, N, () )
		DEFINE_METHOD( N, void, Set, VV, (float32, float32) )

		DEFINE_METHOD( B, b2Vec2, operator -, N, () )
		DEFINE_METHOD( N, void, operator +=, B, (b2Vec2) )
		DEFINE_METHOD( N, void, operator -=, B, (b2Vec2) )
		DEFINE_METHOD( N, void, operator *=, V, (float32) )

		DEFINE_METHOD( V, float32, Length, N, () )
		DEFINE_METHOD( V, float32, LengthSquared, N, () )
		DEFINE_METHOD( V, float32, Normalize, N, () )

		DEFINE_METHOD( V, bool, IsValid, N, () )

		DEFINE_PROPERTY_V( float32, x )
		DEFINE_PROPERTY_V( float32, y )

		DEFINE_TOSTRING(( "{{ {0}, {1} }}", BASEMEMBER(x), BASEMEMBER(y) ))

		static DEFINE_METHOD_CODE( B, b2Vec2, operator +, BB, (b2Vec2, b2Vec2),  { return gcnew MNAME(b2Vec2)( BASEOBJECT(p1) + BASEOBJECT(p2) ); } )
		static DEFINE_METHOD_CODE( B, b2Vec2, operator -, BB, (b2Vec2, b2Vec2),  { return gcnew MNAME(b2Vec2)( BASEOBJECT(p1) - BASEOBJECT(p2) ); } )
		static DEFINE_METHOD_CODE( B, b2Vec2, operator *, VB, (float32, b2Vec2), { return gcnew MNAME(b2Vec2)(             p1 * BASEOBJECT(p2) ); } )
		static DEFINE_METHOD_CODE( V, bool, operator ==, BB, (b2Vec2, b2Vec2),	 { return ( (BASEOBJECT(p1).x == BASEOBJECT(p2).x) && (BASEOBJECT(p1).y == BASEOBJECT(p2).y) ); } )
	};

	DEFINE_STRUCT( b2Mat22 )
	{
		DEFINE_STRUCT_SET( b2Mat22 )
		DEFINE_CONSTRUCTOR( b2Mat22, BB, (b2Vec2, b2Vec2) )
		DEFINE_CONSTRUCTOR( b2Mat22, VVVV, (float32, float32, float32, float32) )
		DEFINE_CONSTRUCTOR( b2Mat22, V, (float32) )

		DEFINE_METHOD( N, void, Set, BB, (b2Vec2, b2Vec2) )
		DEFINE_METHOD( N, void, Set, V, (float32) )
		DEFINE_METHOD( N, void, SetIdentity, N, () )
		DEFINE_METHOD( N, void, SetZero, N, () )
		DEFINE_METHOD( V, float32, GetAngle, N, () )

		DEFINE_METHOD( B, b2Mat22, Invert, N, () )
		DEFINE_METHOD( B, b2Vec2, Solve, B, (b2Vec2) )

		DEFINE_PROPERTY_B( b2Vec2, col1 )
		DEFINE_PROPERTY_B( b2Vec2, col2 )

		DEFINE_TOSTRING(( "{{ ({0}, {1}), ({2}, {3}) }}", BASEMEMBER(col1).x, BASEMEMBER(col1).y, BASEMEMBER(col2).x, BASEMEMBER(col2).y ))
	};

	DEFINE_STRUCT( b2XForm )
	{
		DEFINE_STRUCT_SET( b2XForm )
		DEFINE_CONSTRUCTOR( b2XForm, BB, (b2Vec2, b2Mat22) )

		DEFINE_METHOD( N, void, SetIdentity, N, () )

		DEFINE_PROPERTY_B( b2Vec2, position )
		DEFINE_PROPERTY_B( b2Mat22, R )

		DEFINE_TOSTRING(( "{{ pos=({0}, {1}), R=({2}, {3} | {4}, {5}) }}", BASEMEMBER(position).x, BASEMEMBER(position).y, BASEMEMBER(R).col1.x, BASEMEMBER(R).col1.y, BASEMEMBER(R).col2.x, BASEMEMBER(R).col2.y ))
	};

	DEFINE_STRUCT( b2Sweep )
	{
		DEFINE_STRUCT_SET( b2Sweep )

		DEFINE_METHOD( N, void, GetXForm, PV, (b2XForm, float32) )
		DEFINE_METHOD( N, void, Advance, V, (float32) )

		DEFINE_PROPERTY_B( b2Vec2, localCenter )
		DEFINE_PROPERTY_B( b2Vec2, c0 )
		DEFINE_PROPERTY_B( b2Vec2, c )
		DEFINE_PROPERTY_V( float32, a0 )
		DEFINE_PROPERTY_V( float32, a )
		DEFINE_PROPERTY_V( float32, t0 )
	};

	DEFINE_STATIC( MathFunction )
	{
		DEFINE_FUNCTIONSTATIC( V, float32, b2Dot, BB, (b2Vec2, b2Vec2) )
		
		DEFINE_FUNCTIONSTATIC( V, float32, b2Cross, BB, (b2Vec2, b2Vec2) )
		DEFINE_FUNCTIONSTATIC( B, b2Vec2, b2Cross, BV, (b2Vec2, float32) )
		DEFINE_FUNCTIONSTATIC( B, b2Vec2, b2Cross, VB, (float32, b2Vec2) )

		DEFINE_FUNCTIONSTATIC( B, b2Vec2, b2Mul, BB, (b2Mat22, b2Vec2) )
		DEFINE_FUNCTIONSTATIC( B, b2Vec2, b2MulT, BB, (b2Mat22, b2Vec2) )
		DEFINE_FUNCTIONSTATIC( B, b2Mat22, b2Mul, BB, (b2Mat22, b2Mat22) )
		DEFINE_FUNCTIONSTATIC( B, b2Mat22, b2MulT, BB, (b2Mat22, b2Mat22) )

		DEFINE_FUNCTIONSTATIC( V, float32, b2Distance, BB, (b2Vec2, b2Vec2) )
		DEFINE_FUNCTIONSTATIC( V, float32, b2DistanceSquared, BB, (b2Vec2, b2Vec2) )

		DEFINE_FUNCTIONSTATIC( V, float32, b2Abs, V, (float32) )
		DEFINE_FUNCTIONSTATIC( B, b2Vec2, b2Abs, B, (b2Vec2) )
		DEFINE_FUNCTIONSTATIC( B, b2Mat22, b2Abs, B, (b2Mat22) )

		generic <typename T> where T : IComparable static T MNAME(b2Min)( T a, T b ) { if( a->CompareTo(b) > 0 ) return b; else return a; }
		generic <typename T> where T : IComparable static T MNAME(b2Max)( T a, T b ) { if( a->CompareTo(b) < 0 ) return b; else return a; }
		generic <typename T> where T : IComparable static T MNAME(b2Clamp)( T a, T low, T high ) { return MNAME(b2Max)<T>( low, MNAME(b2Min)<T>( a, high ) ); }
		generic <typename T> where T : value class static void MNAME(b2Swap)( T% a, T% b ) { T w = a; a = b; b = w; }

		DEFINE_FUNCTIONSTATIC( B, b2Vec2, b2Clamp, BBB, (b2Vec2, b2Vec2, b2Vec2) )

		DEFINE_FUNCTIONSTATIC( V, float32, b2Random, N, () )
		DEFINE_FUNCTIONSTATIC( V, float32, b2Random, VV, (float32, float32) )

		DEFINE_FUNCTIONSTATIC( V, uint32, b2NextPowerOfTwo, V, (uint32) )
		DEFINE_FUNCTIONSTATIC( V, bool, b2IsPowerOfTwo, V, (uint32) )
	};
}
