
#pragma once

namespace Box2DLib
{
	DEFINE_STRUCT( b2MassData )
	{
		DEFINE_STRUCT_SET( b2MassData );

		DEFINE_PROPERTY_V( float32, mass );
		DEFINE_PROPERTY_B( b2Vec2, center );
		DEFINE_PROPERTY_V( float32, I );
	};

	DEFINE_STRUCT( b2FilterData )
	{
		DEFINE_STRUCT_SET( b2FilterData );

		DEFINE_PROPERTY_V( uint16, categoryBits );
		DEFINE_PROPERTY_V( uint16, maskBits );
		DEFINE_PROPERTY_V( int16, groupIndex );
	};

	DEFINE_ENUM( b2ShapeType )
	{
		DEFINE_ENUMITEM( e_unknownShape ),
		DEFINE_ENUMITEM( e_circleShape ),
		DEFINE_ENUMITEM( e_polygonShape ),
		DEFINE_ENUMITEM( e_shapeTypeCount ),
	};

	DEFINE_STRUCT( b2ShapeDef )
	{
		DEFINE_BASEACCESS( b2ShapeDef )
		DEFINE_CONSTRUCTOR_DEFAULTDUMMY( b2ShapeDef )
		DEFINE_DELETEBASE_DUMMY()

		DEFINE_PROPERTY_E( b2ShapeType, type );
		DEFINE_PROPERTY_C( IntPtr, userData, static_cast<IntPtr>, static_cast<void*> );
		DEFINE_PROPERTY_V_CODE( int, userDataInt, { return reinterpret_cast<int>(BASEMEMBER(userData)); }, { BASEMEMBER(userData) = reinterpret_cast<void *>(v); } );
		DEFINE_PROPERTY_V( float32, friction );
		DEFINE_PROPERTY_V( float32, restitution );
		DEFINE_PROPERTY_V( float32, density );
		DEFINE_PROPERTY_V( bool, isSensor );
		DEFINE_PROPERTY_B( b2FilterData, filter );
	};

	DEFINE_STRUCT_PROTOTYPE( b2Body );

	DEFINE_STRUCT( b2Shape )
	{
		DEFINE_CONSTRUCTOR_WRAPPER( b2Shape );
		DEFINE_BASEACCESS( b2Shape );
		DEFINE_COPYMETHOD( b2Shape );
		DEFINE_DELETEBASE_DUMMY();

		DEFINE_METHOD( E, b2ShapeType, GetType, N, () );
		DEFINE_METHOD( V, bool, IsSensor, N, () );
		DEFINE_METHOD( N, void, SetFilterData, B, (b2FilterData) );
		DEFINE_METHOD( B, b2FilterData, GetFilterData, N, () );

		DEFINE_METHOD_DECLARE( P, b2Body, GetBody, N, () );

		DEFINE_METHOD( P, b2Shape, GetNext, N, () );
		DEFINE_METHOD( C, IntPtr, GetUserData, N, () );
		DEFINE_METHOD( N, void, SetUserData, I, (IntPtr) );

		virtual DEFINE_METHOD( V, bool, TestPoint, BB, (b2XForm, b2Vec2) );
		// ToDo : virtual bool TestSegment(	const b2XForm& xf, float32* lambda, b2Vec2* normal, const b2Segment& segment, float32 maxLambda) const = 0;
		virtual DEFINE_METHOD( N, void, ComputeAABB, PB, (b2AABB, b2XForm) );
		virtual DEFINE_METHOD( N, void, ComputeSweptAABB, PBB, (b2AABB, b2XForm, b2XForm) );
		virtual DEFINE_METHOD( N, void, ComputeMass, P, (b2MassData) );

		DEFINE_METHOD( V, float32, GetSweepRadius, N, () );
		DEFINE_METHOD( V, float32, GetFriction, N, () );
		DEFINE_METHOD( V, float32, GetRestitution, N, () );

	};
}
