
#pragma once

namespace Box2DLib
{
	DEFINE_STRUCT( b2TimeStep )
	{
		DEFINE_STRUCT_SET( b2TimeStep );

		DEFINE_PROPERTY_V( float32, dt );
		DEFINE_PROPERTY_V( float32, inv_dt );
		DEFINE_PROPERTY_V( float32, dtRatio );
		DEFINE_PROPERTY_V( int32, maxIterations );
		DEFINE_PROPERTY_V( bool, warmStarting );
		DEFINE_PROPERTY_V( bool, positionCorrection );
	};

	
	DEFINE_STRUCT( b2World )
	{
		DEFINE_CONSTRUCTOR_WRAPPER( b2World );
		DEFINE_BASEACCESS( b2World );
		DEFINE_COPYMETHOD( b2World );
		DEFINE_DELETEBASE( b2World );

		DEFINE_CONSTRUCTOR( b2World, BBV, (b2AABB, b2Vec2, bool) );
		// ToDo : ~b2World()

		// ToDo : void SetDestructionListener(b2DestructionListener* listener);
		// ToDo : void SetBoundaryListener(b2BoundaryListener* listener);
		// ToDo : void SetContactFilter(b2ContactFilter* filter);
		// ToDo : void SetContactListener(b2ContactListener* listener);
		// ToDo : void SetDebugDraw(b2DebugDraw* debugDraw);

		DEFINE_METHOD( P, b2Body, CreateBody, P, (b2BodyDef) );
		DEFINE_METHOD( N, void, DestroyBody, P, (b2Body) );

		DEFINE_METHOD( P, b2Joint, CreateJoint, P, (b2JointDef) );
		DEFINE_METHOD( N, void, DestroyJoint, P, (b2Joint) );

		DEFINE_METHOD( P, b2Body, GetGroundBody, N, () );
		DEFINE_METHOD( N, void, Step, VV, (float32, int32) );

		int Query( MNAME(b2AABB)^ aabb, array<MNAME(b2Shape)^>^ shapes, int32 maxCount )
		{
			UNAME(b2Shape)**	shp = new UNAME(b2Shape)*[ maxCount ];
			int	cnt = this->BasePtr->Query( BASEOBJECT(aabb), shp, maxCount );

			for( int x = 0; x < maxCount; x++ )
			{
				if( x < cnt )
				{
					shapes[ x ] = gcnew MNAME(b2Shape)( shp[x] );
				} else
				{
					shapes[ x ] = nullptr;
				}
			}

			delete[] shp;
			return cnt;
		}


		DEFINE_METHOD( P, b2Body, GetBodyList, N, () );
		DEFINE_METHOD( P, b2Joint, GetJointList, N, () );

		DEFINE_METHOD( N, void, Refilter, P, (b2Shape) );
		DEFINE_METHOD( N, void, SetWarmStarting, V, (bool) );
		DEFINE_METHOD( N, void, SetPositionCorrection, V, (bool) );
		DEFINE_METHOD( N, void, SetContinuousPhysics, V, (bool) );

		DEFINE_METHOD( N, void, Validate, N, () );

		DEFINE_METHOD( V, int32, GetProxyCount, N, () );
		DEFINE_METHOD( V, int32, GetPairCount, N, () );
		DEFINE_METHOD( V, int32, GetBodyCount, N, () );
		DEFINE_METHOD( V, int32, GetJointCount, N, () );
		DEFINE_METHOD( V, int32, GetContactCount, N, () );
		
		DEFINE_METHOD( N, void, SetGravity, B, (b2Vec2) );

	};

	DEFINE_CLASSMETHOD( b2Body, P, b2World, GetWorld, N, () );
}
