﻿Option Explicit On
Option Strict On

#Region "参照設定関連"

Imports Microsoft.DirectX
Imports Microsoft.DirectX.Direct3D

Imports System.Drawing
Imports System.Windows.Forms

Imports Box2DLib
Imports Box2DLib.Settings
Imports Box2DLib.MathFunction

#End Region

'//////////////////////////////////////////////////////////////
''' <summary>
''' メインモジュールです。
''' </summary>
''' <remarks></remarks>
Module Main

    '==========================================================

    ''' <summary>表示用フォームを保持します。</summary>
    Private WithEvents mFrm As Box2DForm

    ''' <summary>前回のTickカウントを保持します。</summary>
    Private mLastTick As Integer

    ''' <summary>ブロックの追加を停止させるかどうかの状態を保持します。</summary>
    Private mDropStop As Boolean

    '==========================================================

    ''' <summary>
    ''' プログラムの開始位置です。
    ''' </summary>
    ''' <remarks></remarks>
    Sub Main()
#If DEBUG Then
        'デバッグ時はTraceへの出力をコンソールにも出力する
        Trace.Listeners.Add(New TextWriterTraceListener(System.Console.Out))
#End If

        '表示フォームの準備
        SetupForm()
        If mFrm Is Nothing Then Exit Sub

        'シミュレーションの準備
        SetupWorld()

        '表示
        mFrm.DoModal()

        '表示フォームの破棄
        mFrm.Release()

    End Sub

    '==========================================================

    ''' <summary>
    ''' 表示フォームのセットアップを行います。
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub SetupForm()

        'フォーム作成
        mFrm = New Box2DForm

        '表示スタイルの設定
        mFrm.WindowSize = New Size(800, 600)
        mFrm.FullScreenSize = mFrm.WindowSize
        mFrm.FullScreenStyle = D3DForm.SCREENSTYLECONST.FullScreen

        'セットアップ
        If mFrm.Setup() Then
            mFrm.Release()
            mFrm = Nothing
            Exit Sub
        End If

        '表示設定
        mFrm.CenterOffset = New PointF(0, 0)
        mFrm.Zoom = 10
        mFrm.ZoomSpeed = 1.1F

    End Sub

    ''' <summary>
    ''' 物体のセットアップを行います。
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub SetupWorld()

        'シミュレーションワールドの準備
        mFrm.CreateWorld(100, 100, -10)

        '表示用Direct3Dデバイスの登録
        D3DPrimitive2D.Device = mFrm.Device

        '地面
        Dim b1 As BodyWrapper = mFrm.CreateBody(0.0F, -10.0F)
        b1.DrawStyle.FillColor = Color.Green
        b1.AddBox(12, 1)

        '初期化
        mLastTick = Environment.TickCount() \ 1000

    End Sub

    '==========================================================

    ''' <summary>
    ''' DoModal中にシミュレーションが実行される時に発生します。
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub mFrm_OnStep() Handles mFrm.OnStep

        '新たなブロックの生成
        Dim nt As Integer = Environment.TickCount() \ 1000
        If nt <> mLastTick Then
            '１秒経過

            '状態チェック
            If mDropStop = False Then

                'オブジェクト作成
                DropBlock()

            End If

            mLastTick = nt
        End If

        'テーブルからこぼれたブロックの削除
        BlockCheckAndDelete()

    End Sub

    '==========================================================

    ''' <summary>ブロックを落下させる位置です。</summary>
    Private Const DROPHEIGHT As Single = 10.0F

    ''' <summary>ブロックの基本サイズです。</summary>
    Private Const BLOCKSIZE As Single = 0.5F

    ''' <summary>ブロックの密度です。</summary>
    Private Const BLOCKDENSITY As Single = 10.0F

    ''' <summary>ブロックの摩擦係数です。</summary>
    Private Const BLOCKFRICTION As Single = 0.2F

    ''' <summary>ブロックの反発係数です。</summary>
    Private Const BLOCKRESTITUTION As Single = 0.5F

    ''' <summary>ブロックパーツ(横棒)の座標データです。</summary>
    Private mBlockPartsH() As PointF = {New PointF(-BLOCKSIZE / 2, -BLOCKSIZE / 2), New PointF(BLOCKSIZE / 2, -BLOCKSIZE / 2), New PointF(BLOCKSIZE, 0), New PointF(BLOCKSIZE / 2, BLOCKSIZE / 2), New PointF(-BLOCKSIZE / 2, BLOCKSIZE / 2), New PointF(-BLOCKSIZE, 0)}

    ''' <summary>ブロックパーツ(縦棒)の座標データです。</summary>
    Private mBlockPartsV() As PointF = {New PointF(-BLOCKSIZE / 2, BLOCKSIZE / 2), New PointF(-BLOCKSIZE / 2, -BLOCKSIZE / 2), New PointF(0, -BLOCKSIZE), New PointF(BLOCKSIZE / 2, -BLOCKSIZE / 2), New PointF(BLOCKSIZE / 2, BLOCKSIZE / 2), New PointF(0, BLOCKSIZE)}

    ''' <summary>シェイプの数を保持します。</summary>
    Private mShapeCount As Integer

    ''' <summary>
    ''' 新しいブロックを作成し、落下させます。
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub DropBlock()

        '出力文字列決定
        Dim ss As String = String.Format("{0:d2}:{1:d2}:{2:d2}", Now.Hour, Now.Minute, Now.Second)

        'ブロック作成
        For t As Integer = 0 To ss.Length - 1

            '位置決定
            Dim gx As Single = (t - (ss.Length / 2.0F)) * BLOCKSIZE * 4

            '古いシェイプの削除
            BlockDelete(400)

            'シェイプ登録
            Dim cc As Char = ss(t)
            If ("0"c <= cc) And (cc <= "9") Then
                '数字
                Dim bdy As BodyWrapper = mFrm.CreateBody(gx, DROPHEIGHT, 0)
                bdy.Body.userDataInt = 1
                CreateBlock(bdy, Asc(cc) - Asc("0"c))
            ElseIf cc = ":"c Then
                '：
                Dim bdy1 As BodyWrapper = mFrm.CreateBody(gx, DROPHEIGHT - BLOCKSIZE, 0)
                bdy1.AddCircle(BLOCKSIZE / 2, BLOCKDENSITY * 2, BLOCKFRICTION, BLOCKRESTITUTION)
                bdy1.Body.userDataInt = 1
                Dim bdy2 As BodyWrapper = mFrm.CreateBody(gx, DROPHEIGHT + BLOCKSIZE, 0)
                bdy2.AddCircle(BLOCKSIZE / 2, BLOCKDENSITY * 2, BLOCKFRICTION, BLOCKRESTITUTION)
                bdy2.Body.userDataInt = 1
                mShapeCount += 2
            End If

        Next

    End Sub

    ''' <summary>
    ''' ボディの形状を決定します。
    ''' </summary>
    ''' <param name="Body">対象のボディ</param>
    ''' <param name="Pattern">ボディ形状のパターン番号</param>
    ''' <remarks></remarks>
    Private Sub CreateBlock(ByVal Body As BodyWrapper, ByVal Pattern As Integer)

        '色決定
        'Select Case Pattern
        '    Case 0 : Body.DrawStyle.FillColor = Color.Gray
        '    Case 1 : Body.DrawStyle.FillColor = Color.Blue
        '    Case 2 : Body.DrawStyle.FillColor = Color.Red
        '    Case 3 : Body.DrawStyle.FillColor = Color.Magenta
        '    Case 4 : Body.DrawStyle.FillColor = Color.Green
        '    Case 5 : Body.DrawStyle.FillColor = Color.Cyan
        '    Case 6 : Body.DrawStyle.FillColor = Color.Yellow
        '    Case 7 : Body.DrawStyle.FillColor = Color.White
        '    Case 8 : Body.DrawStyle.FillColor = Color.Silver
        '    Case 9 : Body.DrawStyle.FillColor = Color.Gold
        'End Select

        '形状決定
        Select Case Pattern '上横
            Case 0, 2, 3, 5, 6, 7, 8, 9
                Body.AddPolygon(mBlockPartsH, 0, BLOCKSIZE * 2, Density:=BLOCKDENSITY, Friction:=BLOCKFRICTION, Restitution:=BLOCKRESTITUTION)
                mShapeCount += 1
        End Select
        Select Case Pattern '中横
            Case 2, 3, 4, 5, 6, 8, 9
                Body.AddPolygon(mBlockPartsH, 0, 0, Density:=BLOCKDENSITY, Friction:=BLOCKFRICTION, Restitution:=BLOCKRESTITUTION)
                mShapeCount += 1
        End Select
        Select Case Pattern '下横
            Case 0, 2, 3, 5, 6, 8, 9
                Body.AddPolygon(mBlockPartsH, 0, -BLOCKSIZE * 2, Density:=BLOCKDENSITY, Friction:=BLOCKFRICTION, Restitution:=BLOCKRESTITUTION)
                mShapeCount += 1
        End Select
        Select Case Pattern '上左
            Case 0, 4, 5, 6, 8, 9
                Body.AddPolygon(mBlockPartsV, -BLOCKSIZE, BLOCKSIZE, Density:=BLOCKDENSITY, Friction:=BLOCKFRICTION, Restitution:=BLOCKRESTITUTION)
                mShapeCount += 1
        End Select
        Select Case Pattern '上右
            Case 0, 1, 2, 3, 4, 7, 8, 9
                Body.AddPolygon(mBlockPartsV, BLOCKSIZE, BLOCKSIZE, Density:=BLOCKDENSITY, Friction:=BLOCKFRICTION, Restitution:=BLOCKRESTITUTION)
                mShapeCount += 1
        End Select
        Select Case Pattern '下左
            Case 0, 2, 6, 8
                Body.AddPolygon(mBlockPartsV, -BLOCKSIZE, -BLOCKSIZE, Density:=BLOCKDENSITY, Friction:=BLOCKFRICTION, Restitution:=BLOCKRESTITUTION)
                mShapeCount += 1
        End Select
        Select Case Pattern '下右
            Case 0, 1, 3, 4, 5, 6, 7, 8, 9
                Body.AddPolygon(mBlockPartsV, BLOCKSIZE, -BLOCKSIZE, Density:=BLOCKDENSITY, Friction:=BLOCKFRICTION, Restitution:=BLOCKRESTITUTION)
                mShapeCount += 1
        End Select

    End Sub

    ''' <summary>
    ''' ブロックがテーブルから落ちているかチェックし、落ちたテーブルは削除します。
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub BlockCheckAndDelete()

        '削除対象の検出
        Dim dellist As New List(Of BodyWrapper)
        For Each bdy As BodyWrapper In mFrm.BodyList

            '位置取得
            Dim pos As bb2Vec2 = bdy.Body.GetPosition()

            '削除判断
            If (pos.y < -50.0F) Or (Math.Abs(pos.x) > 50.0F) Then

                '削除リストに追加
                dellist.Add(bdy)

            End If

        Next

        '削除対象の削除
        For Each bdy As BodyWrapper In dellist

            '削除
            BlockDelete(bdy)

        Next

    End Sub

    ''' <summary>
    ''' 対象ボディを削除します。
    ''' </summary>
    ''' <param name="Body">削除するボディ</param>
    ''' <remarks></remarks>
    Private Sub BlockDelete(ByVal Body As BodyWrapper)

        'シェイプ個数削減
        Dim scnt As Integer = Body.GetShapeCount()
        mShapeCount -= scnt

        'ワールドからの削除
        mFrm.BodyList.Remove(Body)
        mFrm.World.DestroyBody(Body.Body)
        Body.Release()

    End Sub

    ''' <summary>
    ''' 指定されたシェイプ数以下になるまで、古いブロックを削除します。
    ''' </summary>
    ''' <param name="MaxCount">最大シェイプ数</param>
    ''' <remarks></remarks>
    Private Sub BlockDelete(ByVal MaxCount As Integer)

        Do While mShapeCount > MaxCount

            '削除対象の検索
            Dim delbdy As BodyWrapper = Nothing
            For Each bdy As BodyWrapper In mFrm.BodyList
                If bdy.Body.userDataInt = 1 Then
                    delbdy = bdy
                    Exit For
                End If
            Next
            If delbdy Is Nothing Then
                Exit Do '削除できるボディなし
            End If

            '削除
            BlockDelete(delbdy)

        Loop

    End Sub

    '==========================================================

    ''' <summary>
    ''' キー押下時の処理です。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub mFrm_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles mFrm.KeyDown

        Select Case e.KeyCode
            Case Keys.S
                '追加停止/解除
                mDropStop = Not mDropStop

        End Select

    End Sub

    '==========================================================

    ''' <summary>
    ''' 描画処理です。
    ''' </summary>
    ''' <param name="Dev"></param>
    ''' <remarks>
    ''' Box2DFormでのボディ等の描画が終わった後に発生します。
    ''' </remarks>
    Private Sub mFrm_OnDrawEvent(ByVal Dev As Microsoft.DirectX.Direct3D.Device) Handles mFrm.OnDrawEvent

        'キーヘルプの描画
        mFrm.DrawText(8, 48, "P Key : pause")
        mFrm.DrawText(8, 64, "S Key : drop stop")
        mFrm.DrawText(8, 80, "Mouse Left Button  : pickup")
        mFrm.DrawText(8, 96, "Mouse Right Button : scroll")
        mFrm.DrawText(8, 112, "Mouse Wheel : Zoom")

    End Sub

End Module
