﻿Option Explicit On
Option Strict On

#Region "参照設定関連"

Imports Microsoft.DirectX
Imports Microsoft.DirectX.Direct3D

Imports System.Drawing
Imports System.Windows.Forms

Imports Box2DLib
Imports Box2DLib.Settings
Imports Box2DLib.MathFunction

#End Region

'//////////////////////////////////////////////////////////////
''' <summary>
''' Box2DのShapeオブジェクトをラップするクラスです。
''' </summary>
''' <remarks></remarks>
Public Class ShapeWrapper

    '==========================================================

#Region "初期化・破棄関連"

    '内部データ

    ''' <summary>対象のBox2Dシェイプを保持します。</summary>
    Protected mShape As bb2Shape

    'メソッド

    ''' <summary>
    ''' クラスを初期化します。
    ''' </summary>
    ''' <param name="Shape">対象シェイプ</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal Shape As bb2Shape, ByVal Style As DrawStyleData)

        '引数チェック
        If Shape Is Nothing Then Throw New Exception("対象のShapeが指定されていません。")

        '引数保存
        mShape = Shape
        mDrawStyle = Style

        '描画用プリミティブの準備
        CreatePrimitive()

    End Sub

    ''' <summary>
    ''' 内部データを開放します。
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub Release()

        mShape = Nothing

        If mPrimitive IsNot Nothing Then
            mPrimitive.Release()
            mPrimitive = Nothing
        End If

    End Sub

#End Region

#Region "描画用プリミティブ関連"

    '内部データ

    ''' <summary>描画スタイルを保持します。</summary>
    Protected mDrawStyle As DrawStyleData

    ''' <summary>描画用プリミティブを保持します。</summary>
    Protected mPrimitive As D3DPrimitive2D

    'メソッド

    ''' <summary>
    ''' 描画用プリミティブを生成します。
    ''' </summary>
    ''' <remarks></remarks>
    Protected Sub CreatePrimitive()

        'チェック
        If mShape Is Nothing Then Exit Sub

        'タイプ別にプリミティブを生成
        Select Case mShape.GetType()
            Case bb2ShapeType.e_polygonShape
                CreatePolygonPrimitive(mDrawStyle.FillColor, mDrawStyle.LineColor, mDrawStyle.PosZ)

            Case bb2ShapeType.e_circleShape
                CreateCirclePrimitive(mDrawStyle.FillColor, mDrawStyle.LineColor, mDrawStyle.PosZ)

        End Select

    End Sub

    ''' <summary>
    ''' 多角形プリミティブを作成します。
    ''' </summary>
    ''' <param name="FillColor">塗り潰し色</param>
    ''' <param name="BorderColor">輪郭線色</param>
    ''' <param name="PosZ">ｚ座標</param>
    ''' <remarks></remarks>
    Protected Sub CreatePolygonPrimitive(ByVal FillColor As Color, ByVal BorderColor As Color, ByVal PosZ As Single)

        '頂点の取得
        Dim pnt As New List(Of PointF)
        Dim pshp As bb2PolygonShape = bb2PolygonShape.CastFrom(mShape)
        Dim cnt As Integer = pshp.GetVertexCount()
        For p As Integer = 0 To cnt - 1
            Dim pos As bb2Vec2 = pshp.Vertices(p)
            pnt.Add(New PointF(pos.x, pos.y))
        Next

        'プリミティブ作成
        mPrimitive = D3DPrimitive2DSolid.CreatePolygon(pnt, FillColor, BorderColor, , , posz)

    End Sub

    ''' <summary>
    ''' 円形プリミティブを作成します。
    ''' </summary>
    ''' <param name="FillColor">塗り潰し色</param>
    ''' <param name="BorderColor">輪郭線色</param>
    ''' <param name="PosZ">ｚ座標</param>
    ''' <remarks></remarks>
    Protected Sub CreateCirclePrimitive(ByVal FillColor As Color, ByVal BorderColor As Color, ByVal PosZ As Single)

        '半径の取得
        Dim cshp As bb2CircleShape = bb2CircleShape.CastFrom(mShape)
        Dim r As Single = cshp.GetRadius()

        'プリミティブ作成
        mPrimitive = D3DPrimitive2DSolid.CreateCircle(r, FillColor, BorderColor, , , PosZ)

    End Sub

#End Region

#Region "描画関係"

    ''' <summary>
    ''' シェイプを描画します。
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub Draw()

        '状態チェック
        If mPrimitive Is Nothing Then Exit Sub

        '描画
        mPrimitive.Draw()

    End Sub

#End Region

End Class
