﻿Option Explicit On
Option Strict On

#Region "参照設定"

'このクラスモジュールを使用する場合、下記の.NETコンポーネントを
'「プロジェクト」－「参照の追加」で追加する必要があります。
'   Microsoft.DirectX
'   Microsoft.DirectX.Direct3D
'   Microsoft.DirectX.Direct3DX
Imports Microsoft.DirectX
Imports Microsoft.DirectX.Direct3D

'コンソールアプリの場合、下記コンポーネントも必要となります。
'  System.Drawing
'  System.Windows.Forms
Imports System.Drawing
Imports System.Windows.Forms

#End Region

'//////////////////////////////////////////////////////////////
''' <summary>
''' Direct3Dによる表示を行うフォームクラスです。
''' </summary>
''' <remarks></remarks>
Public Class D3DForm
    Inherits Form       'フォームを継承

#Region "表示設定用データ/プロパティ"

    '--------------------------

    ''' <summary>画面の色数を表す定数です。</summary> 
    Public Enum COLORDEPTHCONST As Integer

        ''' <summary>16ビットカラーモード(R5G6B5)</summary> 
        Bit16 = Microsoft.DirectX.Direct3D.Format.R5G6B5

        ''' <summary>32ビットカラーモード(X8R8G8B8)</summary> 
        Bit32 = Microsoft.DirectX.Direct3D.Format.X8R8G8B8

    End Enum

    ''' <summary>表示スタイルを表す定数です。</summary>
    Public Enum SCREENSTYLECONST

        ''' <summary>ウィンドウモード</summary>
        Windowed = 0

        ''' <summary>最大表示モード</summary>
        Maximized = 1

        ''' <summary>フルスクリーンモード(占有モード)</summary>
        FullScreen = 2

    End Enum

    '--------------------------

    ''' <summary>使用するアダプタの番号を保持します。</summary> 
    Protected mAdapterIndex As Integer = 0

    ''' <summary>フルスクリーンモード時の表示サイズを保持します。</summary> 
    Protected mFullScreenSize As Size = New Size(640, 480)

    ''' <summary>フルスクリーンモード時の色深度を保持します。</summary> 
    Protected mFullScreenColorDepth As COLORDEPTHCONST = COLORDEPTHCONST.Bit16

    ''' <summary>フルスクリーンモード時の最低リフレッシュレートを保持します。</summary> 
    Protected mFullScreenRefreshRate As Integer = 0

    ''' <summary>ウィンドウモード時の表示サイズを保持します。</summary> 
    Protected mWindowSize As Size = New Size(640, 480)

    ''' <summary>ウィンドウモード時のユーザによるサイズ変更許可設定を保持します。</summary> 
    Protected mWindowResizable As Boolean = False

    ''' <summary>現在の表示スタイルを保持します。</summary>
    Protected mScreenStyle As SCREENSTYLECONST = SCREENSTYLECONST.Windowed

    ''' <summary>全体表示時の表示スタイルを保持します。</summary>
    Protected mFullScreenStyle As SCREENSTYLECONST = SCREENSTYLECONST.Maximized

    '--------------------------

    ''' <summary>
    ''' 使用するディスプレイアダプタの番号を取得・設定します。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>Setupを呼び出す前に設定する必要があります。</remarks>
    Public Property AdapterIndex() As Integer
        Get
            Return mAdapterIndex
        End Get
        Set(ByVal value As Integer)
            mAdapterIndex = value
        End Set
    End Property

    ''' <summary>
    ''' フルスクリーンモード時の解像度を取得・設定します。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>変更を有効にするには、Setup又はChangeScreenを呼び出す必要があります。</remarks>
    Public Property FullScreenSize() As Size
        Get
            Return mFullScreenSize
        End Get
        Set(ByVal value As Size)
            mFullScreenSize = value
        End Set
    End Property

    ''' <summary>
    ''' フルスクリーンモードでの最低リフレッシュレートを取得・設定します。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>変更を有効にするには、Setup又はChangeScreenを呼び出す必要があります。</remarks>
    Public Property FullScreenRefreshRate() As Integer
        Get
            Return mFullScreenRefreshRate
        End Get
        Set(ByVal value As Integer)
            mFullScreenRefreshRate = value
        End Set
    End Property

    ''' <summary>
    ''' フルスクリーンモードでの色深度を取得・設定します。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>変更を有効にするには、Setup又はChangeScreenを呼び出す必要があります。</remarks>
    Public Property FullScreenColorDepth() As COLORDEPTHCONST
        Get
            Return mFullScreenColorDepth
        End Get
        Set(ByVal value As COLORDEPTHCONST)
            mFullScreenColorDepth = value
        End Set
    End Property

    ''' <summary>
    ''' ウィンドウモード時の解像度を取得・設定します。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>変更を有効にするには、Setup又はChangeScreenを呼び出す必要があります。</remarks>
    Public Property WindowSize() As Size
        Get
            Return mWindowSize
        End Get
        Set(ByVal value As Size)
            mWindowSize = value
        End Set
    End Property

    ''' <summary>
    ''' ウィンドウモード時のユーザによるサイズ変更を許可するかどうかを取得・設定します。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>変更を有効にするには、Setup又はChangeScreenを呼び出す必要があります。</remarks>
    Public Property WindowResizable() As Boolean
        Get
            Return mWindowResizable
        End Get
        Set(ByVal value As Boolean)
            mWindowResizable = value
        End Set
    End Property

    ''' <summary>
    ''' 現在のスクリーンスタイルを取得します。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property ScreenStyle() As SCREENSTYLECONST
        Get
            Return mScreenStyle
        End Get
    End Property

    ''' <summary>
    ''' 全体表示時の表示スタイルを取得・設定します。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>Windowedを指定した場合、全体表示はできなくなります。</remarks>
    Public Property FullScreenStyle() As SCREENSTYLECONST
        Get
            Return mFullScreenStyle
        End Get
        Set(ByVal value As SCREENSTYLECONST)
            mFullScreenStyle = value
        End Set
    End Property

    ''' <summary>
    ''' 現在ウィンドウモードかどうかを取得・設定します。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property Windowed() As Boolean
        Get
            If mScreenStyle = SCREENSTYLECONST.Windowed Then Return True
            Return False
        End Get
        Set(ByVal value As Boolean)

            If value Then
                'ウィンドウモードへ
                mScreenStyle = SCREENSTYLECONST.Windowed
            Else
                '全体表示(最大表示モードorフルスクリーンモード)へ
                '(mFullScreenStyleがWindowedの場合を除く)
                mScreenStyle = mFullScreenStyle
            End If

        End Set
    End Property

#End Region

#Region "制御用データ/プロパティ"

    ''' <summary>現在使用中のDirect3Dデバイスを保持します。</summary> 
    Protected WithEvents mDev As Microsoft.DirectX.Direct3D.Device

    ''' <summary>リサイズイベントを無視するかどうかの設定を保持します。</summary> 
    Protected mIgnoreResizeEvent As Boolean = False

    ''' <summary>終了処理中かどうかの状態を保持します。</summary>
    Protected mShutdownProcessing As Boolean = False

    ''' <summary>
    ''' 現在使用中のDirect3Dデバイスオブジェクトを取得します。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Device() As Microsoft.DirectX.Direct3D.Device
        Get
            Return mDev
        End Get
    End Property

    ''' <summary>
    ''' 現在終了処理中かどうかを返します。
    ''' </summary>
    ''' <value></value>
    ''' <returns>Shutdownメソッドによる終了処理中の場合True、その他はFalse</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property ShutdownProcessing() As Boolean
        Get
            Return mShutdownProcessing
        End Get
    End Property

#End Region

#Region "初期化関連"

    ''' <summary>
    ''' Direct3D関連の初期化を行います。
    ''' </summary>
    ''' <returns>成功した場合False、失敗した場合True</returns>
    ''' <remarks>初期化できなかった場合、ダイアログメッセージが表示されます。</remarks>
    Public Overridable Function Setup() As Boolean

        Try

            '現在のD3Dデバイスを破棄
            Release()

            'フォームの設定
            SetupForm()

            '表示設定の準備
            Dim pp As PresentParameters = CreatePresentParameters()

            'D3Dデバイスの作成
            For createtype As Integer = 0 To 2
                Try

                    'デバイス作成(下記の順に作成を試みる)
                    Select Case createtype
                        Case 0  '頂点処理：ハード　　ラスタライザ：ハード
                            mDev = New Device(mAdapterIndex, DeviceType.Hardware, Me.Handle, CreateFlags.HardwareVertexProcessing, pp)
                        Case 1  '頂点処理：ソフト　　ラスタライザ：ハード
                            mDev = New Device(mAdapterIndex, DeviceType.Hardware, Me.Handle, CreateFlags.SoftwareVertexProcessing, pp)
                        Case 2  '頂点処理：ソフト　　ラスタライザ：ソフト
                            mDev = New Device(mAdapterIndex, DeviceType.Software, Me.Handle, CreateFlags.SoftwareVertexProcessing, pp)
                    End Select

                    '作成成功
                    If mDev IsNot Nothing Then Exit For

                Catch ex As Exception
                    '失敗(リトライする)
                End Try
            Next
            If mDev Is Nothing Then Throw New Exception("Direct3Dデバイスを作成できません。")

            'システムフォント作成
            SetupSystemFont(SYSTEMFONTNAME, SYSTEMFONTSIZE)

            'サイズ変更時の処理
            SetupView()

            '基本描画設定
            SetupRenderState()

        Catch ex As Exception
            'セットアップ失敗

            'メッセージ表示
            MsgBox("Direct3Dの初期化に失敗しました。" + vbCrLf + ex.Message)

            'デバイス破棄
            Release()

            'セットアップ失敗
            Return True

        End Try

        '成功
        mShutdownProcessing = False
        Return False
    End Function

    ''' <summary>
    ''' Direct3D関連のオブジェクトを開放します。
    ''' </summary>
    ''' <remarks></remarks>
    Public Overridable Sub Release()

        If mDev IsNot Nothing Then
            mDev.Dispose()
            mDev = Nothing
        End If

    End Sub

    ''' <summary>
    ''' 状態に合わせたDirect3Dデバイスの表示設定を作成します。
    ''' </summary>
    ''' <returns>作成された表示設定</returns>
    ''' <remarks></remarks>
    Protected Overridable Function CreatePresentParameters() As PresentParameters

        Dim pp As New PresentParameters

        '共通設定
        pp.SwapEffect = SwapEffect.Discard            'バッファ切り替え時に以前の内容は破棄する
        pp.EnableAutoDepthStencil = True              '深度バッファを使用する
        pp.AutoDepthStencilFormat = DepthFormat.D16   '深度バッファは16bit

        'スクリーンモード毎の設定
        If mScreenStyle = SCREENSTYLECONST.FullScreen Then
            'フルスクリーンモード

            pp.Windowed = False

            'ディスプレイモードの検索
            Dim foundformat As Boolean = False
            If mAdapterIndex < Manager.Adapters.Count Then
                For Each dm As Microsoft.DirectX.Direct3D.DisplayMode In Manager.Adapters(mAdapterIndex).SupportedDisplayModes
                    If ((mFullScreenColorDepth = COLORDEPTHCONST.Bit16) And (dm.Format = Format.R5G6B5)) Or _
                       ((mFullScreenColorDepth = COLORDEPTHCONST.Bit32) And (dm.Format = Format.X8R8G8B8)) Then
                        If (dm.Width = mFullScreenSize.Width) And (dm.Height = mFullScreenSize.Height) Then
                            If mFullScreenRefreshRate <= dm.RefreshRate Then
                                '発見
                                pp.BackBufferWidth = mFullScreenSize.Width
                                pp.BackBufferHeight = mFullScreenSize.Height
                                pp.BackBufferFormat = dm.Format
                                pp.FullScreenRefreshRateInHz = dm.RefreshRate
                                foundformat = True
                                Exit For
                            End If
                        End If
                    End If
                Next
            End If
            If Not foundformat Then Throw New Exception(String.Format("指定されたディスプレイモードは使用できません。アダプタ＝{0}, 画面サイズ＝{1}x{2}, リフレッシュレート＝{3}(以上)", mAdapterIndex, mFullScreenSize.Width, mFullScreenSize.Height, mFullScreenRefreshRate))

        Else
            'ウィンドウモード

            pp.Windowed = True
            pp.BackBufferFormat = Format.Unknown
            pp.FullScreenRefreshRateInHz = 0

        End If

        Return pp
    End Function

    ''' <summary>
    ''' フォームのサイズやスタイルを設定に合わせて変更します。
    ''' </summary>
    ''' <remarks></remarks>
    Protected Overridable Sub SetupForm()

        'リサイズイベント無効
        Dim ire As Boolean = mIgnoreResizeEvent
        mIgnoreResizeEvent = True

        '表示スタイル毎の処理
        Select Case mScreenStyle

            Case SCREENSTYLECONST.Windowed       'ウィンドウモード
                'フォームのボーダースタイルを設定
                If mWindowResizable Then
                    Me.FormBorderStyle = Windows.Forms.FormBorderStyle.Sizable      'サイズ変更可
                    Me.MaximizeBox = True   '最大化ボタン：有効
                Else
                    Me.FormBorderStyle = Windows.Forms.FormBorderStyle.FixedSingle  'サイズ変更不可
                    Me.MaximizeBox = False  '最大化ボタン：無効
                End If

                Me.WindowState = FormWindowState.Normal     'ウィンドウ表示
                Me.ClientSize = mWindowSize                 '表示サイズ

            Case SCREENSTYLECONST.Maximized    '最大表示モード
                Me.FormBorderStyle = Windows.Forms.FormBorderStyle.None 'ボーダーなし
                Me.WindowState = FormWindowState.Maximized              '最大化表示

            Case SCREENSTYLECONST.FullScreen     'フルスクリーンモード
                Me.FormBorderStyle = Windows.Forms.FormBorderStyle.None 'ボーダーなし
                Me.ClientSize = mFullScreenSize             '表示サイズ

        End Select

        'リサイズイベント復帰
        mIgnoreResizeEvent = ire

    End Sub

    ''' <summary>
    ''' スクリーンの表示スタイルを変更します。
    ''' </summary>
    ''' <remarks></remarks>
    Public Overridable Sub ChangeScreen(Optional ByVal ForceReset As Boolean = False)

        '状態チェック
        If mDev Is Nothing Then Exit Sub 'セットアップされてない

        Try
            'フォームの設定
            SetupForm()

            '新しい表示設定の作成
            Dim pp As PresentParameters = CreatePresentParameters()

            'デバイスのリセット
            mDev.Reset(pp)

            'サイズ変更時の処理
            SetupView()

            '基本描画設定
            SetupRenderState()

        Catch ex As Exception

            'ウィンドウを閉じる
            Me.Close()

            'メッセージ表示
            MsgBox("切替に失敗しました。" + vbCrLf + ex.Message)

        End Try

    End Sub

    ''' <summary>
    ''' 基本的な描画条件等を設定します。
    ''' </summary>
    ''' <remarks></remarks>
    Public Overridable Sub SetupRenderState()

        mDev.RenderState.CullMode = Cull.None   'カリングモード：なし
        mDev.RenderState.Lighting = False       'ライティング：なし
        mDev.RenderState.ZBufferEnable = False  'Ｚバッファ：無効

    End Sub

    ''' <summary>
    ''' 表示サイズが変更された時の処理です。
    ''' </summary>
    ''' <remarks></remarks>
    Public Overridable Sub SetupView()

    End Sub

    ''' <summary>
    ''' 終了処理を開始します。
    ''' </summary>
    ''' <remarks></remarks>
    Public Overridable Sub Shutdown()

        mShutdownProcessing = True
        Me.Close()

    End Sub

#End Region

#Region "キー操作関連"

    '内部データ

    ''' <summary>Escキーによる終了設定を保持します。</summary>
    Protected mEnableEscape As Boolean = True

    ''' <summary>Alt+Enterによるスクリーン切替許可設定を保持します。</summary>
    Protected mEnableAltEnter As Boolean = True

    ''' <summary>Ctrl+Enterによる全体表示時のスタイル変更許可を保持します。</summary>
    Protected mEnableCtrlEnter As Boolean = True

    'プロパティ

    ''' <summary>
    ''' Escキーで終了するかどうかを取得・設定します。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property EnableEscapeKey() As Boolean
        Get
            Return mEnableEscape
        End Get
        Set(ByVal value As Boolean)
            mEnableEscape = value
        End Set
    End Property

    ''' <summary>
    ''' Alt+Enterでウィンドウ表示/全体表示を切り替えられるかどうかを取得・設定します。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property EnableAltEnter() As Boolean
        Get
            Return mEnableAltEnter
        End Get
        Set(ByVal value As Boolean)
            mEnableAltEnter = value
        End Set
    End Property

    ''' <summary>
    ''' Alt+Ctrlで全体表示スタイルを切り替えられるかどうかを取得・設定します。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property EnableCtrlEnter() As Boolean
        Get
            Return mEnableCtrlEnter
        End Get
        Set(ByVal value As Boolean)
            mEnableCtrlEnter = value
        End Set
    End Property


    'イベントハンドラ

    ''' <summary>
    ''' フォームでのキー入力時の処理です。
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overrides Sub OnKeyDown(ByVal e As System.Windows.Forms.KeyEventArgs)

        Select Case e.KeyCode
            Case Keys.Escape
                'Escキー押下

                If mEnableEscape Then

                    '終了
                    Shutdown()
                    Exit Sub

                End If

            Case Keys.Enter
                If e.Alt Then
                    If mEnableAltEnter Then
                        'Alt + Enterキー押下を許可

                        'スクリーンスタイル更新
                        Dim ss As SCREENSTYLECONST = mScreenStyle
                        Me.Windowed = Not Me.Windowed

                        'スクリーン更新
                        If ss <> mScreenStyle Then
                            ChangeScreen()
                        End If

                    End If
                End If

                If e.Control Then
                    If mEnableCtrlEnter Then
                        'Ctrl + Enterキー押下を許可

                        '全体表示時のスタイルを変更
                        If mFullScreenStyle = SCREENSTYLECONST.FullScreen Then
                            mFullScreenStyle = SCREENSTYLECONST.Maximized
                        Else
                            mFullScreenStyle = SCREENSTYLECONST.FullScreen
                        End If

                    End If
                End If

        End Select

        'その他
        MyBase.OnKeyDown(e)
    End Sub

#End Region

#Region "描画関連"

    '内部データ

    ''' <summary>クリア色を保持します。</summary> 
    Protected mClearColor As Color = Color.DarkBlue

    'プロパティ

    ''' <summary>
    ''' クリア色を取得・設定します。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property ClearColor() As Color
        Get
            Return mClearColor
        End Get
        Set(ByVal value As Color)
            mClearColor = value
        End Set
    End Property

    'イベント

    ''' <summary>
    ''' 描画が必要な時に発生します。
    ''' </summary>
    ''' <param name="Dev">レンダリングデバイス</param>
    ''' <remarks></remarks>
    Public Event OnDrawEvent(ByVal Dev As Microsoft.DirectX.Direct3D.Device)

    'メソッド

    ''' <summary>
    ''' フレーム毎の処理を行います。
    ''' </summary>
    ''' <remarks></remarks>
    Public Overridable Sub Render()

        '状態チェック
        If mDev Is Nothing Then Exit Sub '初期化されてない
        If Me.WindowState = FormWindowState.Minimized Then Exit Sub '最小化表示

        'デバイス状態の確認
        Dim rc As Integer
        If mDev.CheckCooperativeLevel(rc) = False Then
            Select Case rc
                Case ResultCode.DeviceLost
                    'デバイスは消失していて、リセットもできない状態

                    '待機
                    System.Threading.Thread.Sleep(10)
                    Return

                Case ResultCode.DeviceNotReset
                    'デバイスは消失しているが、リセットできる状態

                    'リセット
                    mDev.Reset(mDev.PresentationParameters)

                    'サイズ変更処理
                    SetupView()

                    '基本的な描画設定
                    SetupRenderState()

                Case Else
                    'その他(終了)
                    Me.Close()
                    Return

            End Select
        End If

        'メイン描画処理
        Draw()

        '表示
        mDev.Present()

        'フレームレート更新
        UpdateFramerate()

    End Sub

    ''' <summary>
    ''' 描画処理を行います。
    ''' </summary>
    ''' <remarks></remarks>
    Protected Overridable Sub Draw()

        'クリア
        Clear()

        '描画開始
        mDev.BeginScene()

        '描画前の処理
        BeforeDraw()

        '描画処理
        OnDraw()

        '描画後の処理
        AfterDraw()

        '描画終了
        mDev.EndScene()

    End Sub

    ''' <summary>
    ''' フレームバッファのクリアを行います。
    ''' </summary>
    ''' <remarks></remarks>
    Protected Overridable Sub Clear()

        mDev.Clear(ClearFlags.Target Or ClearFlags.ZBuffer, mClearColor, 1, 0)

    End Sub

    ''' <summary>
    ''' 描画イベントの直前に実行される処理です。
    ''' </summary>
    ''' <remarks></remarks>
    Protected Overridable Sub BeforeDraw()

    End Sub

    ''' <summary>
    ''' 描画処理を行います。
    ''' </summary>
    ''' <remarks></remarks>
    Protected Overridable Sub OnDraw()

        '描画イベント発生
        RaiseEvent OnDrawEvent(mDev)

    End Sub

    ''' <summary>
    ''' 描画イベントの直後に実行される処理です。
    ''' </summary>
    ''' <remarks></remarks>
    Protected Overridable Sub AfterDraw()

        'フレームレート描画
        If mShowFrameRate Then
            DrawFrameRate()
        End If

    End Sub

    ''' <summary>
    ''' ワールド変換行列をリセット(単位行列)します。
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub ResetWorldTransform()
        mDev.Transform.World = Matrix.Identity
    End Sub

#End Region

#Region "システムフォント関連"

    '定数

    ''' <summary>システム情報表示用フォント名です。</summary> 
    Public Const SYSTEMFONTNAME As String = "ＭＳ ゴシック"

    ''' <summary>システム情報表示用フォントサイズです。</summary> 
    Public Const SYSTEMFONTSIZE As Integer = 12

    '内部データ

    ''' <summary>システム情報描画用フォントを保持します。</summary> 
    Protected mDrawTextFont As Microsoft.DirectX.Direct3D.Font

    ''' <summary>システム情報描画時の色を保持します。</summary> 
    Protected mDrawTextColor As Color = Color.White

    'プロパティ

    ''' <summary>
    ''' システム情報描画時の色を取得・設定します。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property SystemTextColor() As Color
        Get
            Return mDrawTextColor
        End Get
        Set(ByVal value As Color)
            mDrawTextColor = value
        End Set
    End Property

    'メソッド

    ''' <summary>
    ''' システム情報描画用フォント作成します。
    ''' </summary>
    ''' <param name="FontName">フォント名</param>
    ''' <param name="FontSize">フォントサイズ</param>
    ''' <remarks></remarks>
    Protected Sub SetupSystemFont(ByVal FontName As String, ByVal FontSize As Integer)

        'フォント情報の準備
        Dim fd As Microsoft.DirectX.Direct3D.FontDescription = New FontDescription
        fd.FaceName = FontName
        fd.Height = FontSize

        'フォント作成
        mDrawTextFont = New Microsoft.DirectX.Direct3D.Font(mDev, fd)

    End Sub

    ''' <summary>
    ''' 文字列を描画します。
    ''' </summary>
    ''' <param name="PosX">描画位置ｘ</param>
    ''' <param name="PosY">描画位置</param>
    ''' <param name="Fmt">書式</param>
    ''' <param name="Obj">引数</param>
    ''' <remarks>システム情報描画用フォントを使って描画されます。</remarks>
    Public Overridable Sub DrawText(ByVal PosX As Integer, ByVal PosY As Integer, ByVal Fmt As String, ByVal ParamArray Obj As Object())

        If mDrawTextFont IsNot Nothing Then
            mDrawTextFont.DrawText(Nothing, String.Format(Fmt, Obj), PosX, PosY, mDrawTextColor)
        End If

    End Sub

#End Region

#Region "フレームレート関連"

    '定数

    ''' <summary>フレームレートの更新間隔(ミリ秒)です。</summary> 
    Public Const FRAMERATEUPDATEINTERVAL As Integer = 1000

    ''' <summary>フレームレート表示時に描画する情報を選択する定数です。</summary>
    <Flags()> _
    Public Enum SHOWFRAMERATEFLAGS

        ''' <summary>何も表示しない</summary>
        None = 0

        ''' <summary>フレームレートを表示する</summary>
        FrameRate = 1

        ''' <summary>サイズを表示する</summary>
        Size = 2

        ''' <summary>スクリーンスタイルを表示する</summary>
        Style = 4

        ''' <summary>全体表示スタイルを表示する</summary>
        FullScreenStyle = 8

        ''' <summary>切替キーヘルプを表示する</summary>
        KeyHelp = 128

    End Enum

    '内部データ

    ''' <summary>フレームレート描画設定を保持します。</summary>
    Protected mShowFrameRate As Boolean = True

    ''' <summary>フレームレート描画で表示する情報フラグを保持します。</summary>
    Protected mShowFrameRateFlag As SHOWFRAMERATEFLAGS = SHOWFRAMERATEFLAGS.FrameRate Or SHOWFRAMERATEFLAGS.Size Or SHOWFRAMERATEFLAGS.Style Or SHOWFRAMERATEFLAGS.FullScreenStyle Or SHOWFRAMERATEFLAGS.KeyHelp

    ''' <summary>フレームレートを保持します。</summary> 
    Protected mFrameRate As Double = 0

    ''' <summary>前回のフレームレート計算からのフレーム数を保持します。</summary> 
    Protected mFrameCount As Integer = 0

    ''' <summary>前回のフレームレート計算の時刻を保持します。</summary> 
    Protected mLastFrameTime As Integer = 0

    'プロパティ

    ''' <summary>
    ''' フレームレートを描画するかどうかを取得・設定します。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property ShowFrameRate() As Boolean
        Get
            Return mShowFrameRate
        End Get
        Set(ByVal value As Boolean)
            mShowFrameRate = value
        End Set
    End Property

    ''' <summary>
    ''' フレームレート表示時に描画する情報を取得・設定します。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property ShowFrameRateFlag() As SHOWFRAMERATEFLAGS
        Get
            Return mShowFrameRateFlag
        End Get
        Set(ByVal value As SHOWFRAMERATEFLAGS)
            mShowFrameRateFlag = value
        End Set
    End Property

    'メソッド

    ''' <summary>
    ''' フレームレートを更新します。
    ''' </summary>
    ''' <remarks></remarks>
    Protected Overridable Sub UpdateFramerate()

        'フレーム数カウント
        mFrameCount += 1

        '現在時刻取得
        Dim tm As Integer = System.Environment.TickCount And Integer.MaxValue

        '前回計算時刻のチェック
        If mLastFrameTime < 0 Then
            mLastFrameTime = tm
        End If

        '前回計算時からの経過時間取得
        Dim dt As Integer = tm - mLastFrameTime
        If dt < 0 Then
            mLastFrameTime = tm
        ElseIf dt >= FRAMERATEUPDATEINTERVAL Then
            mFrameRate = 1000 * mFrameCount / dt
            mLastFrameTime = tm
            mFrameCount = 0
        End If

    End Sub

    ''' <summary>
    ''' フレームレートを描画します。
    ''' </summary>
    ''' <remarks></remarks>
    Protected Overridable Sub DrawFrameRate()

        Dim tt As New System.Text.StringBuilder

        '描画情報作成
        If (mShowFrameRateFlag And SHOWFRAMERATEFLAGS.FrameRate) <> 0 Then
            If (mShowFrameRateFlag And SHOWFRAMERATEFLAGS.KeyHelp) <> 0 Then
                tt.Append("FPS:")
            End If
            tt.AppendFormat("{0:0.0}  ", mFrameRate)
        End If
        If (mShowFrameRateFlag And SHOWFRAMERATEFLAGS.Size) <> 0 Then
            If (mShowFrameRateFlag And SHOWFRAMERATEFLAGS.KeyHelp) <> 0 Then
                tt.Append("Size:")
            End If
            Dim sz As Size = Me.ClientSize
            tt.AppendFormat("{0},{1}  ", sz.Width, sz.Height)
        End If
        If (mShowFrameRateFlag And SHOWFRAMERATEFLAGS.Style) <> 0 Then
            If (mShowFrameRateFlag And SHOWFRAMERATEFLAGS.KeyHelp) <> 0 Then
                tt.Append("Mode:")
            End If
            Select Case mScreenStyle
                Case SCREENSTYLECONST.Windowed
                    tt.Append("Window")
                Case SCREENSTYLECONST.Maximized
                    tt.Append("MaxWindow")
                Case SCREENSTYLECONST.FullScreen
                    tt.Append("FullScreen")
            End Select
            If (mShowFrameRateFlag And SHOWFRAMERATEFLAGS.KeyHelp) <> 0 Then
                tt.Append("(change Alt+Enter)")
            End If
            tt.Append("  ")
        End If
        If (mShowFrameRateFlag And SHOWFRAMERATEFLAGS.FullScreenStyle) <> 0 Then
            tt.Append("FullMode:")
            Select Case mFullScreenStyle
                Case SCREENSTYLECONST.Windowed
                    tt.Append("Window")
                Case SCREENSTYLECONST.Maximized
                    tt.Append("MaxWindow")
                Case SCREENSTYLECONST.FullScreen
                    tt.Append("FullScreen")
            End Select
            If (mShowFrameRateFlag And SHOWFRAMERATEFLAGS.KeyHelp) <> 0 Then
                tt.Append("(change Ctrl+Enter)")
            End If
            tt.Append("  ")
        End If

        '描画
        If mDrawTextFont IsNot Nothing Then
            DrawText(0, 0, tt.ToString)
        End If

    End Sub

#End Region

#Region "サイズ変更関連"

    ''' <summary>
    ''' フォームサイズが変更された時の処理です。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub DXForm_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize

        'チェック
        If mIgnoreResizeEvent Then Exit Sub
        If mDev Is Nothing Then Exit Sub

        'ウィンドウ状態毎の処理
        Select Case Me.WindowState
            Case FormWindowState.Maximized  '最大化されている場合

                '最大表示モードへ
                mScreenStyle = SCREENSTYLECONST.Maximized

                'スクリーン変更
                ChangeScreen()

            Case FormWindowState.Minimized  'アイコン化されている場合

                'なにもしない

            Case FormWindowState.Normal     '通常ウィンドウ表示の場合

                If mScreenStyle <> SCREENSTYLECONST.FullScreen Then

                    'ウィンドウモードへ
                    mScreenStyle = SCREENSTYLECONST.Windowed

                    'ウィンドウサイズ更新
                    Me.WindowSize = Me.ClientSize

                    'スクリーン変更
                    ChangeScreen()

                End If

            Case Else   'その他

                'なにもしない

        End Select


    End Sub

#End Region

#Region "デバイスロスト関連"

    'イベントハンドラ

    ''' <summary>
    ''' デバイスロスト時のイベントハンドラです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub mDev_DeviceLost(ByVal sender As Object, ByVal e As System.EventArgs) Handles mDev.DeviceLost

        'デバイスロスト処理
        OnDeviceLost()

    End Sub

    ''' <summary>
    ''' デバイスリセット字のイベントハンドラです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub mDev_DeviceReset(ByVal sender As Object, ByVal e As System.EventArgs) Handles mDev.DeviceReset

        'デバイスリセット処理
        OnDeviceReset()

    End Sub

    'メソッド

    ''' <summary>
    ''' デバイスロスト時の処理です。
    ''' </summary>
    ''' <remarks></remarks>
    Public Overridable Sub OnDeviceLost()

        Debug.Print("Device Lost")

    End Sub

    ''' <summary>
    ''' デバイスリセット時の処理です。
    ''' </summary>
    ''' <remarks></remarks>
    Public Overridable Sub OnDeviceReset()

        Debug.Print("Device Reset")

    End Sub

#End Region

End Class
