﻿Option Explicit On
Option Strict On

#Region "参照設定関連"

Imports Microsoft.DirectX
Imports Microsoft.DirectX.Direct3D

Imports System.Drawing
Imports System.Windows.Forms

Imports Box2DLib
Imports Box2DLib.Settings
Imports Box2DLib.MathFunction

#End Region

'//////////////////////////////////////////////////////////////
''' <summary>
''' メインモジュールです。
''' </summary>
''' <remarks></remarks>
Module Main

    '==========================================================

    ''' <summary>表示用フォームを保持します。</summary>
    Private WithEvents mFrm As Box2DForm

    '==========================================================
    ''' <summary>
    ''' プログラムの開始位置です。
    ''' </summary>
    ''' <remarks></remarks>
    Sub Main()
#If DEBUG Then
        'デバッグ時はTraceへの出力をコンソールにも出力する
        Trace.Listeners.Add(New TextWriterTraceListener(System.Console.Out))
#End If

        '表示フォームの準備
        SetupForm()
        If mFrm Is Nothing Then Exit Sub

        'シミュレーションの準備
        SetupWorld()

        '表示
        mFrm.DoModal()

        '表示フォームの破棄
        mFrm.Release()

    End Sub

    '==========================================================

    ''' <summary>
    ''' 表示フォームのセットアップを行います。
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub SetupForm()

        'フォーム作成
        mFrm = New Box2DForm

        '表示スタイルの設定
        mFrm.WindowSize = New Size(800, 600)
        mFrm.FullScreenSize = mFrm.WindowSize
        mFrm.FullScreenStyle = D3DForm.SCREENSTYLECONST.FullScreen

        'セットアップ
        If mFrm.Setup() Then
            mFrm.Release()
            mFrm = Nothing
            Exit Sub
        End If

        '表示設定
        mFrm.CenterOffset = New PointF(0, -7)
        mFrm.Zoom = 9
        mFrm.ZoomSpeed = 1.1F

    End Sub

    ''' <summary>
    ''' 物体のセットアップを行います。
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub SetupWorld()

        'シミュレーションワールドの準備
        mFrm.CreateWorld(100, 100, -10)

        '表示用Direct3Dデバイスの登録
        D3DPrimitive2D.Device = mFrm.Device

        '地面
        Dim b1 As BodyWrapper = mFrm.CreateBody(0.0F, -10.0F)
        b1.DrawStyle.FillColor = Color.Green
        b1.AddBox(50, 10)
        '下記は上記をBox2D本来の生成方法で作成した場合の手順です。
        'Dim s1def As New bb2PolygonDef
        's1def.SetAsBox(50.0F, 10.0F)
        'Dim b1def As New bb2BodyDef
        'b1def.position.Set(0.0F, -10.0F)
        'Dim d1 As bb2Body = mFrm.World.CreateBody(b1def)
        'd1.CreateShape(s1def)
        'd1.SetMassFromShapes()
        'Dim w1 As New Box2DBodyWrapper(d1)     'ボディリストへの追加
        'w1.ViewConfig.FillColor = Color.Green
        'mFrm.BodyList.Add(w1)

        '上段の床
        Dim bd As BodyWrapper = mFrm.CreateBody(-1.5F, 10.0F)
        bd.DrawStyle.FillColor = Color.Green
        bd.AddBox(6.0F, 0.25F)

        '上段に配置されるドミノ
        For t As Integer = 0 To 9
            bd = mFrm.CreateBody(-6.0F + t, 11.25F)
            bd.DrawStyle.FillColor = Color.Yellow
            bd.AddBox(0.1F, 1.0F, Density:=20.0F, Friction:=0.1F)
        Next

        '中段の斜め床
        bd = mFrm.CreateBody(1.0F, 6.0F)
        bd.DrawStyle.FillColor = Color.Green
        bd.AddBox(7.0F, 0.25F, Angle:=0.3F)

        '左側の壁
        Dim b2 As BodyWrapper = mFrm.CreateBody(-7.0F, 4.0F)
        b2.DrawStyle.FillColor = Color.Green
        b2.AddBox(0.25F, 1.5F)

        'シーソーの板
        Dim b3 As BodyWrapper = mFrm.CreateBody(-0.9F, 1.0F, -0.15F)
        b3.DrawStyle.FillColor = Color.Cyan
        b3.AddBox(6.0F, 0.125F, Density:=10.0F)

        '地面とシーソーの接続
        mFrm.CreateRevoluteJoint(b1, b3, -2.0F, 1.0F, Color.Transparent)
        '下記は上記をBox2D本来の生成方法で作成した場合の手順です。
        'Dim jd As New bb2RevoluteJointDef
        'Dim anchor As New bb2Vec2
        'anchor.Set(-2.0F, 1.0F)
        'jd.Initialize(b1.Body, b3.Body, anchor)
        'jd.collideConnected = True
        'mFrm.World.CreateJoint(jd)

        '振り子
        Dim b4 As BodyWrapper = mFrm.CreateBody(-10.0F, 15.0F)
        b4.DrawStyle.FillColor = Color.LightGray
        b4.AddBox(0.25F, 0.25F, Density:=10.0F)

        '壁と振り子の接続
        mFrm.CreateRevoluteJoint(b2, b4, -7.0F, 15.0F, Color.LightGray)

        'かご
        Dim b5 As BodyWrapper = mFrm.CreateBody(6.5F, 3.0F)
        b5.DrawStyle.FillColor = Color.LightGreen
        b5.AddBox(1.0F, 0.1F, 0.0F, -0.9F, Density:=10.0F, Friction:=0.1)
        b5.AddBox(0.1F, 1.0F, -0.9F, 0.0F, Density:=10.0F, Friction:=0.1)
        b5.AddBox(0.1F, 1.0F, 0.9F, 0.0F, Density:=10.0F, Friction:=0.1)

        '地面とかごの接続
        mFrm.CreateRevoluteJoint(b1, b5, 6.0F, 2.0F, Color.Transparent)

        'かごの蓋
        Dim b6 As BodyWrapper = mFrm.CreateBody(6.5F, 4.1F)
        b6.DrawStyle.FillColor = Color.LightBlue
        b6.AddBox(1.0F, 0.1F, Density:=30.0F, Friction:=0.2F)

        'かごとかごの蓋の接続
        mFrm.CreateRevoluteJoint(b5, b6, 7.5F, 4.0F, Color.Transparent)

        'つっかえ棒
        Dim b7 As BodyWrapper = mFrm.CreateBody(7.4F, 1.0F)
        b7.DrawStyle.FillColor = Color.LightPink
        b7.AddBox(0.1F, 1.0F, Density:=10.0F)

        'シーソーとつっかえ棒の接続
        Dim djd As New bb2DistanceJointDef
        djd.body1 = b3.Body
        djd.body2 = b7.Body
        djd.localAnchor1.Set(6.0F, 0.0F)
        djd.localAnchor2.Set(0.0F, -1.0F)
        Dim d As bb2Vec2 = djd.body2.GetWorldPoint(djd.localAnchor2) - djd.body1.GetWorldPoint(djd.localAnchor1)
        djd.length = d.Length
        Dim j5 As bb2Joint = mFrm.World.CreateJoint(djd)
        mFrm.CreateJoint(j5, Color.White)

        'かごの中の玉
        Dim cd As New bb2CircleDef
        cd.radius = 0.2F
        cd.density = 10
        For t As Integer = 0 To 3
            bd = mFrm.CreateBody(5.9F + 2.0F * cd.radius * t, 2.4F)
            bd.DrawStyle.FillColor = Color.White
            bd.AddShape(cd)
        Next

    End Sub

    '==========================================================

    ''' <summary>
    ''' キー押下時の処理です。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub mFrm_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles mFrm.KeyDown

        Select Case e.KeyCode
            Case Keys.R
                'リセット
                SetupWorld()

        End Select

    End Sub

    '==========================================================

    ''' <summary>
    ''' 描画処理です。
    ''' </summary>
    ''' <param name="Dev"></param>
    ''' <remarks>
    ''' Box2DFormでのボディ等の描画が終わった後に発生します。
    ''' </remarks>
    Private Sub mFrm_OnDrawEvent(ByVal Dev As Microsoft.DirectX.Direct3D.Device) Handles mFrm.OnDrawEvent

        'キーヘルプの描画
        mFrm.DrawText(8, 32, "R Key : Reset")
        mFrm.DrawText(8, 48, "P Key : Pause")
        mFrm.DrawText(8, 64, "Mouse Left Button  : Pickup")
        mFrm.DrawText(8, 80, "Mouse Right Button : Scroll")
        mFrm.DrawText(8, 96, "Mouse Wheel : Zoom")

    End Sub

End Module
