﻿Option Explicit On
Option Strict On

'Box2DLib.DLLを参照設定する必要があります。
Imports Box2DLib

Module Module1

    'これはBox2Dを使用してシミュレーションを構築/実行するシンプルな使用例です。
    '地面を表す箱と落下する小さな箱を作成します。
    Sub Main()

        'シミュレーションワールドのサイズを定義します。
        Dim worldAABB As New bb2AABB
        worldAABB.lowerBound.Set(-100.0F, -100.0F)
        worldAABB.upperBound.Set(100.0F, 100.0F)

        '重力ベクタを定義します。
        Dim gravity As New bb2Vec2(0.0F, -10.0F)

        '物体はスリープ状態になれることとします。
        Dim doSleep As Boolean = True

        'ワールドオブジェクトを生成します。
        '(ワールドオブジェクトは物体を保持してシミュレーションを行います。）
        Dim world As New bb2World(worldAABB, gravity, doSleep)

        '地面用の物体定義を生成します。
        Dim groundBodyDef As New bb2BodyDef
        groundBodyDef.position.Set(0.0F, -10.0F)

        '地面用の物体定義を引数にCreateBodyを呼び出すと
        'シミュレーションワールドに地面を表す物体(ボディー)が追加されます。
        Dim graoundBody As bb2Body = world.CreateBody(groundBodyDef)

        '地面物体用の形状定義を生成します。
        Dim groundShapeDef As New bb2PolygonDef

        '半分のサイズを指定して形状定義を「ボックス」にします。
        groundShapeDef.SetAsBox(50.0F, 10.0F)

        '地面物体(ボディー)に形状(ボックスのシェイプ)を与えます。
        graoundBody.CreateShape(groundShapeDef)

        '落下する箱物体を位置を指定して生成します。
        Dim bodyDef As New bb2BodyDef
        bodyDef.position.Set(0.0F, 4.0F)
        Dim body As bb2Body = world.CreateBody(bodyDef)

        '落下箱物体の形状定義を生成します。
        Dim shapedef As New bb2PolygonDef
        shapedef.SetAsBox(1.0F, 1.0F)

        '箱の密度(density)を0以外にします。これにより移動するようになります。
        shapedef.density = 1.0F

        '摩擦係数(friction)を設定します。
        shapedef.friction = 0.3F

        '落下箱物体に形状を与えます。
        body.CreateShape(shapedef)

        '塊属性を現在の形状から自動生成します。
        body.SetMassFromShapes()

        'シミュレーションの準備として、
        'タイムステップを60Hz、繰り返し回数を10にします。
        '通常はこれで十分です。
        Dim timeStep As Single = 1.0F / 60.0F
        Dim iterations As Integer = 10

        '６０回繰り返します。
        For i As Integer = 0 To 59

            'シミュレーションを１ステップ行います。
            '一般的にタイムステップと繰り返し回数は固定にした方がよいです。
            world.Step(timeStep, iterations)

            '落下箱物体の位置と角度を出力します。
            Dim position As bb2Vec2 = body.GetPosition()
            Dim angle As Single = body.GetAngle()
            Console.WriteLine("{0,3:f} {1,3:f} {2,3:f}", position.x, position.y, angle)

        Next

    End Sub

End Module
