﻿Option Explicit On
Option Strict On

'Box2DLibを使用するには、「Box2DLib.DLL」を参照設定する必要があります。
Imports Box2DLib


'//////////////////////////////////////////////////////////////
Public Class Form1

    ''' <summary>画面表示用のビットマップを保持します。</summary>
    Private mBMP As Bitmap

    ''' <summary>シミュレーション動作中かどうかを示す値を保持します。</summary>
    Private mRun As Boolean

    ''' <summary>シミュレーションワールドを保持します。</summary>
    Private mWorld As bb2World

    '==========================================================

    ''' <summary>
    ''' シミュレーションワールドを表示するときの設定情報が格納される構造体です。
    ''' </summary>
    ''' <remarks>シミュレーションワールド座標を表示座標に変換する際に使用されます。</remarks>
    Private Structure ViewSetting

        ''' <summary>表示エリアのサイズ(ピクセル)</summary>
        Public ShowSize As SizeF

        ''' <summary>表示する時の中心座標(シミュレーションワールド座標)</summary>
        Public Center As PointF

        ''' <summary>表示する時の縦幅(シミュレーションワールド座標)</summary>
        Public ViewSize As Single

        ''' <summary>
        ''' シミュレーション空間の座標を表示用座標に変換します。
        ''' </summary>
        ''' <param name="PosX">シミュレーション空間座標ｘ</param>
        ''' <param name="PosY">シミュレーション空間座標ｙ</param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Function ConvertWorldToScreen(ByVal PosX As Single, ByVal PosY As Single, Optional ByVal ScaleOnly As Boolean = False) As PointF

            '計算準備
            Dim wy As Single = ViewSize
            Dim wx As Single = ViewSize * ShowSize.Width / ShowSize.Height 'Pic_Viewのアスペクト比に合わせて計算

            '計算
            Dim ret As New PointF
            If ScaleOnly Then
                'シミュレーション空間内の距離などをPic_Viewの描画単位に変換する

                ret.X = ShowSize.Width / 2 * PosX / wx
                ret.Y = ShowSize.Height / 2 * PosY / wy

            Else
                'シミュレーション空間の( cx-wx , cy-wy )～( cx+wx , cy-wy )の範囲が
                'Pic_Viewの中に表示されるように変換する

                ret.X = ShowSize.Width / 2 * (PosX - Center.X) / wx + ShowSize.Width / 2
                ret.Y = ShowSize.Height / 2 * (PosY - Center.Y) / wy + ShowSize.Height / 2
                ret.Y = ShowSize.Height - ret.Y '上下変換

            End If

            Return ret
        End Function

    End Structure

    ''' <summary>表示用設定を保持します。</summary>
    Private mView As New ViewSetting

    '==========================================================

    ''' <summary>
    ''' フォームロード時の処理です。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        '表示用PictureBoxの準備
        mBMP = New Bitmap(Pic_View.ClientSize.Width, Pic_View.ClientSize.Height)
        Pic_View.Image = mBMP

        '初期化
        mView.ShowSize = Pic_View.ClientSize
        Cmb_World.SelectedIndex = 0
        SetupWorld()
        DrawAll()

    End Sub

    ''' <summary>
    ''' フォームが閉じられる時の処理です。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing

        If mRun Then
            '動作中

            MsgBox("動作中です。終了ボタンを押してから閉じて下さい。")
            e.Cancel = True
        End If

    End Sub

    '==========================================================

    ''' <summary>
    ''' 初期化ボタン押下時の処理です。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub Btn_Initialize_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_Initialize.Click

        'シミュレーションワールドのセットアップ
        SetupWorld()

        '表示
        DrawAll()

    End Sub

    ''' <summary>
    ''' シミュレーションワールドのセットアップを行います。
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub SetupWorld()

        '物体の配置
        Select Case Cmb_World.Text
            Case "HelloWorld"
                SetupWorld_HelloWorld()

            Case "箱と球"
                SetupWorld_BoxAndBall()

            Case "スライダークランク"
                SetupWorld_SliderCrank()

            Case Else
                Debug.Print(Cmb_World.SelectedText)

        End Select

    End Sub


    ''' <summary>
    ''' Box2D産婦のHello Worldの配置を行います。
    ''' (一部値を変えてある所があります。)
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub SetupWorld_HelloWorld()

        'シミュレーションワールドのサイズを定義します。
        Dim worldAABB As New bb2AABB
        worldAABB.lowerBound.Set(-100.0F, -100.0F)
        worldAABB.upperBound.Set(100.0F, 100.0F)

        '重力ベクタを定義します。
        Dim gravity As New bb2Vec2(0.0F, -10.0F)

        '物体はスリープ状態になれることとします。
        Dim doSleep As Boolean = True

        'ワールドオブジェクトを生成します。
        '(ワールドオブジェクトは物体を保持してシミュレーションを行います。）
        mWorld = New bb2World(worldAABB, gravity, doSleep)

        '地面用の物体定義を生成します。
        Dim groundBodyDef As New bb2BodyDef
        groundBodyDef.position.Set(0.0F, -10.0F)

        '地面用の物体定義を引数にCreateBodyを呼び出すと
        'シミュレーションワールドに地面を表す物体(ボディー)が追加されます。
        Dim graoundBody As bb2Body = mWorld.CreateBody(groundBodyDef)

        '地面物体用の形状定義を生成します。
        Dim groundShapeDef As New bb2PolygonDef

        '半分のサイズを指定して形状定義を「ボックス」にします。
        groundShapeDef.SetAsBox(50.0F, 10.0F)

        '地面物体(ボディー)に形状(ボックスのシェイプ)を与えます。
        graoundBody.CreateShape(groundShapeDef)

        '落下する箱物体を位置を指定して生成します。
        Dim bodyDef As New bb2BodyDef
        bodyDef.position.Set(0.0F, 8.0F) '4 -> 8
        Dim body As bb2Body = mWorld.CreateBody(bodyDef)

        '落下箱物体の形状定義を生成します。
        Dim shapedef As New bb2PolygonDef
        shapedef.SetAsBox(1.0F, 1.0F)

        '箱の密度(density)を0以外にします。これにより移動するようになります。
        shapedef.density = 1.0F

        '摩擦係数(friction)を設定します。
        shapedef.friction = 0.3F

        '落下箱物体に形状を与えます。
        body.CreateShape(shapedef)

        '塊属性を現在の形状から自動生成します。
        body.SetMassFromShapes()

        '----------------------

        '表示設定
        mView.Center = New PointF(0.0F, 4.0F)
        mView.ViewSize = 7.0F

    End Sub

    ''' <summary>
    ''' 箱と球を配置します。
    ''' （HelloWorldに球を追加しただけです。）
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub SetupWorld_BoxAndBall()

        'HelloWorld
        SetupWorld_HelloWorld()

        '球の配置
        Dim bf As New bb2BodyDef
        bf.position.Set(0.5, 4)
        Dim bd As bb2Body
        bd = mWorld.CreateBody(bf)
        Dim sd As New bb2CircleDef
        sd.radius = 1
        sd.density = 1
        sd.friction = 0.3
        bd.CreateShape(sd)
        bd.SetMassFromShapes()

    End Sub

    ''' <summary>
    ''' スライダークランクを配置します。
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub SetupWorld_SliderCrank()

        'ワールド生成
        CreateBasicWorld()

        '物体の配置
        'このデモはBox2Dデモの Slider Crank を参考にしています。

        '地面
        Dim sd As New bb2PolygonDef
        sd.SetAsBox(50.0F, 10.0F)
        Dim bd As New bb2BodyDef
        bd.position.Set(0.0F, -10.0F)
        Dim ground As bb2Body = mWorld.CreateBody(bd)

        'クランク(回転する棒)
        sd.SetAsBox(0.5F, 2.0F)
        sd.density = 1.0F

        Dim rjd As New bb2RevoluteJointDef

        Dim prevBody As bb2Body = ground

        bd.position.Set(0.0F, 7.0F)
        Dim body As bb2Body = mWorld.CreateBody(bd)
        body.CreateShape(sd)
        body.SetMassFromShapes()

        rjd.Initialize(prevBody, body, New bb2Vec2(0.0F, 5.0F))
        rjd.motorSpeed = 1.0F * Settings.bb2_pi
        rjd.maxMotorTorque = 10000.0F
        rjd.enableMotor = True
        Dim jnt1 As bb2RevoluteJoint = bb2RevoluteJoint.CastFrom(mWorld.CreateJoint(rjd))

        prevBody = body

        'フロウ(回転する棒
        sd.SetAsBox(0.5F, 4.0F)
        bd.position.Set(0.0F, 13.0F)
        body = mWorld.CreateBody(bd)
        body.CreateShape(sd)
        body.SetMassFromShapes()

        rjd.Initialize(prevBody, body, New bb2Vec2(0.0F, 9.0F))
        rjd.enableMotor = False
        mWorld.CreateJoint(rjd)

        prevBody = body

        'ピストン(上下運動するボックス)
        sd.SetAsBox(1.5F, 1.5F)
        bd.position.Set(0.0F, 17.0F)
        body = mWorld.CreateBody(bd)
        body.CreateShape(sd)
        body.SetMassFromShapes()

        rjd.Initialize(prevBody, body, New bb2Vec2(0.0F, 17.0F))
        mWorld.CreateJoint(rjd)

        Dim pjd As New bb2PrismaticJointDef
        pjd.Initialize(ground, body, New bb2Vec2(0.0F, 17.0F), New bb2Vec2(0.0F, 1.0F))

        pjd.maxMotorForce = 1000.0F
        pjd.enableMotor = True

        Dim jnt2 As bb2PrismaticJoint = bb2PrismaticJoint.CastFrom(mWorld.CreateJoint(pjd))

        ''ペイロード
        sd.density = 2.0F
        bd.position.Set(0.0F, 23.0F)
        body = mWorld.CreateBody(bd)
        body.CreateShape(sd)
        body.SetMassFromShapes()

        '----------------------

        '表示設定
        mView.Center = New PointF(0.0F, 14.0F)
        mView.ViewSize = 16.0F

    End Sub

    ''' <summary>
    ''' 基本的なシミュレーションワールドを生成します。
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub CreateBasicWorld()

        'シミュレーションワールドのサイズを定義します。
        '物体がワールドの外にあってもシミュレーションは行われますが遅くなります。
        Dim worldAABB As New bb2AABB
        worldAABB.lowerBound.Set(-100.0F, -100.0F)
        worldAABB.upperBound.Set(100.0F, 100.0F)

        '重力ベクタを定義します。
        Dim gravity As New bb2Vec2(0.0F, -10.0F)

        '物体はスリープ状態になれることとします。
        Dim doSleep As Boolean = True

        'ワールドオブジェクトを生成します。
        '(ワールドオブジェクトは物体を保持してシミュレーションを行います。）
        mWorld = New bb2World(worldAABB, gravity, doSleep)

    End Sub

    '==========================================================

    ''' <summary>
    ''' 開始ボタン押下時の処理です。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub Btn_Start_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_Start.Click

        'ループ中チェック
        If mRun Then Exit Sub '既に動作中

        'ボタンの状態変更
        Btn_Start.Enabled = False
        Btn_End.Enabled = True
        Chk_Pause.Enabled = True
        Btn_Close.Enabled = False

        '停止ボタンが押されるまでループ
        Dim lt As Integer = Environment.TickCount()
        Dim nt As Integer
        mRun = True
        Me.Text = "シミュレーション実行中"
        Do While mRun

            '時間調整
            If Chk_Wait.Checked Then
                nt = Environment.TickCount()
                If nt - lt < 16 Then
                    Continue Do
                End If
            End If
            lt = nt

            'シミュレーション
            If mWorld IsNot Nothing Then
                If Chk_Pause.Checked = False Then

                    '進める
                    mWorld.Step(1.0F / 60.0F, 10)

                End If
            End If

            '描画
            DrawAll()

            'メッセージ処理
            Application.DoEvents()
        Loop
        Me.Text = "シミュレーション終了"

        'ボタンの状態変更
        Btn_End.Enabled = False
        Chk_Pause.Enabled = False
        Btn_Start.Enabled = True
        Btn_Close.Enabled = True

    End Sub

    ''' <summary>
    ''' 終了ボタン押下時の処理です。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub Btn_End_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_End.Click

        'シミュレーション動作を終了させる
        mRun = False

    End Sub

    ''' <summary>
    ''' 閉じるボタン押下時の処理です。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub Btn_Close_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_Close.Click

        Me.Close()

    End Sub

    '==========================================================

    ''' <summary>
    ''' 描画します。
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub DrawAll()

        '状態チェック
        If mWorld Is Nothing Then Exit Sub '初期化されていない

        '描画開始
        Dim g As Graphics = Graphics.FromImage(mBMP)
        g.Transform.Reset()
        g.Clear(Color.White)

        '全てのボディを列挙し、表示する
        Dim bdy As bb2Body = mWorld.GetBodyList() '先頭のボディ取得
        Do While bdy IsNot Nothing

            'ボディの位置取得
            Dim pos As bb2Vec2 = bdy.GetPosition()

            'ボディを構成する全てのシェイプを列挙し、表示する
            Dim shp As bb2Shape = bdy.GetShapeList() '先頭のシェイプ取得
            Do While shp IsNot Nothing

                'シェイプタイプ別の描画処理
                Select Case shp.GetType()
                    Case bb2ShapeType.e_polygonShape    '多角形
                        Dim poly As bb2PolygonShape = bb2PolygonShape.CastFrom(shp)

                        '頂点座標の取得
                        Dim cnt As Integer = poly.GetVertexCount() '頂点数
                        Dim pnt(cnt - 1) As PointF
                        For t As Integer = 0 To cnt - 1

                            '現在のシミュレーション空間での頂点座標を計算
                            Dim pp As bb2Vec2 = bdy.GetWorldPoint(poly.Vertices(t))

                            '描画空間の座標に変換
                            pnt(t) = mView.ConvertWorldToScreen(pp.x, pp.y)
                        Next

                        '描画
                        g.DrawPolygon(Pens.Black, pnt)

                    Case bb2ShapeType.e_circleShape     '円形
                        Dim cir As bb2CircleShape = bb2CircleShape.CastFrom(shp)

                        '半径取得
                        Dim r As Single = cir.GetRadius()

                        '描画空間の座標に変換
                        Dim pnt As PointF = mView.ConvertWorldToScreen(pos.x, pos.y)
                        Dim pr As PointF = mView.ConvertWorldToScreen(r, r, True)

                        '描画
                        g.DrawEllipse(Pens.Black, pnt.X - pr.X, pnt.Y - pr.Y, pr.X * 2, pr.Y * 2)

                End Select


                '次のシェイプへ
                shp = shp.GetNext()
            Loop

            '次のボディへ
            bdy = bdy.GetNext()
        Loop

        '描画終了
        g.Dispose()
        Pic_View.Refresh()

    End Sub

End Class
